#include "client/client.h"


using namespace dsm;

//DECLARE_string(graph_dir);
DECLARE_string(result_dir);
DECLARE_int64(num_nodes);
DECLARE_double(portion);
DECLARE_double(vert_num);

struct PagerankIterateKernel : public IterateKernel<int, float, vector<int> > {
    float zero;


    PagerankIterateKernel() : zero(0){}

    void read_data(string& line, int& k, vector<int>& data){
        string linestr(line);
        int pos = linestr.find("\t");
        if(pos == -1) return;
        
        int source = boost::lexical_cast<int>(linestr.substr(0, pos));

        vector<int> linkvec;
        string links = linestr.substr(pos+1);
        //cout<<"links:"<<links<<endl;
        int spacepos = 0;
        while((spacepos = links.find_first_of(" ")) != links.npos){
            int to;
            if(spacepos > 0){
                to = boost::lexical_cast<int>(links.substr(0, spacepos));
                //cout<<"to:"<<to<<endl;
            }
            links = links.substr(spacepos+1);
            linkvec.push_back(to);
        }

        k = source;
        data = linkvec;
    }
    
    void read_checkpoint(string& line, int& k, float& v){ 
        string linestr(line);
        int pos = linestr.find("\t");
        if(pos == -1) return;
        
        k = boost::lexical_cast<int>(linestr.substr(0, pos));
        v = boost::lexical_cast<float>(linestr.substr(pos+1));
        //v = 0.0;
        //cout << "read_ck v" << k << "=" << v << endl;
    }
    
    void read_special_checkpoint(string& line, int& k, float& delta, float& v){ 
        string linestr(line);
        int pos = linestr.find("\t");
        if(pos == -1) return;
        k = boost::lexical_cast<int>(linestr.substr(0, pos));
        
        string links = linestr.substr(pos+1);
        pos = links.find("\t");
        if (pos == -1) return;
        delta = boost::lexical_cast<float>(links.substr(0, pos));
        
        v = boost::lexical_cast<float>(links.substr(pos+1));
    }

    void init_c(const int& k, float& delta,vector<int>& data){
        //delta = 0.000416146*0.2; // 1/#_v * 0.2
        //delta = 0.2 * 0.2;
        delta = (1/FLAGS_vert_num) * 0.2;
    }

    void init_v(const int& k,float& v,vector<int>& data){
        v=0;  
    }
    
    void accumulate(float& a, const float& b){
        a = a + b;
    }

    void priority(float& pri, const float& value, const float& delta){
            pri = abs(delta);
    }

    /** Generate messages (i.e., output) for all outgoing neighbors. */
    void g_func(const int& k,const float& delta, const float&value, const vector<int>& data, vector<pair<int, float> >* output){
        if (delta == 0) {
            return;
        }

        float outv = (delta*0.8) / (int) data.size();
        for(vector<int>::const_iterator it=data.begin(); it!=data.end(); it++){
            int target = *it;
            output->push_back(make_pair(target, outv));
            //cout << " send " << outv << " from v" << k << " to v" << target << endl;
        }
    }

    const float& default_v() const {
            return zero;
    }
  
    //add by wzg, only used in fault-tolerance, send original messages instead of delta_v
    void fc_func(const int& k,const float& delta,const float& value, const vector<int>& data, vector<pair<int, float> >* output) {
        if (value == 0) {
            return;
        }
        
        float outv = value / (int) data.size();
        for(vector<int>::const_iterator it=data.begin(); it!=data.end(); it++) {
            int target = *it;
            output->push_back(make_pair(target, outv));
            //cout << "fc_send " << outv << " from v" << k << " to v" << target << endl;
        }
    }
  
    //add by wzg, only used in fault-tolerance
    void fc_recover_delta_v(const int& k, float& delta, float& value) {
        //float tmp_v = 0.8*delta + 0.000416146*0.2;
        //float tmp_v = 0.8*delta + 0.2*0.2;
        float tmp_v = 0.8*delta + (1/FLAGS_vert_num)*0.2;
        delta = tmp_v - value;
        //cout << "fc_v" << k << ", old=" << value << ", new=" << tmp_v << ", delta=" << delta << endl;
    }
};

/**
 * K=int, V=float
 */
  struct PagerankTermChecker : public TermChecker<int, float> {
    double last;
    double curr;
    
    PagerankTermChecker(){
        last = -std::numeric_limits<double>::max();
        curr = 0;
    }

    double set_curr(){
        return curr;
    }
    
    double estimate_prog(LocalTableIterator<int, float>* statetable){
        double partial_curr = 0;
        double defaultv = statetable->defaultV();
        while(!statetable->done()){
            bool cont = statetable->Next();
            if(!cont) break;
            //statetable->Next();
            //cout << statetable->key() << "\t" << statetable->value2() << endl;
           if(statetable->value2() != defaultv){
                partial_curr += static_cast<double>(statetable->value2());
            }
            //partial_curr += abs(static_cast<double>(statetable->value2()));
        }
        return partial_curr;
    }
    
    bool terminate(vector<double> local_reports){
        curr = 0;
        vector<double>::iterator it;
        for(it=local_reports.begin(); it!=local_reports.end(); it++){
                curr += *it;
        }
        
        VLOG(0) << "terminate check : last=" << last << " current=" << curr << " diff=" << abs(1.0-curr) << " threshold=" << FLAGS_termcheck_threshold;
        cout << "terminate check : last=" << last << " current=" << curr << " diff=" << abs(1.0-curr) << endl;
        if(abs(1.0-curr) <= FLAGS_termcheck_threshold){
            //VLOG(0)<<"FLAGS_termcheck_threshold: "<<FLAGS_termcheck_threshold << "currtrue: "<< curr;
            return true;
        }else{
            last = curr;
            //VLOG(0)<<"FLAGS_termcheck_threshold: "<<FLAGS_termcheck_threshold << "curr: "<< curr;
            return false;
        }
    }
  };

static int Pagerank(ConfigData& conf) {
    MaiterKernel<int, float, vector<int> >* kernel = new MaiterKernel<int, float, vector<int> >(
                                        conf, FLAGS_num_nodes, FLAGS_portion, FLAGS_result_dir,
                                        new Sharding::Mod,
                                        new PagerankIterateKernel,
                                        new PagerankTermChecker);
    
    
    kernel->registerMaiter();

    if (!StartWorker(conf)) {
        Master m(conf);
        m.run_maiter(kernel);
    }
    
    delete kernel;
    return 0;
}

REGISTER_RUNNER(Pagerank);
